'use strict';

var os = require('os');
var _ = require('lodash');
var path = require('path');
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');
var mkdirp = require('mkdirp');

module.exports = yeoman.generators.Base.extend({
  prompting: function () {
    var done = this.async();

    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the ' + chalk.red('lxgo') + ' yeoman generator! Scaffolding your golang projects.'
    ));

    var prompts = [
      {
        name: 'app_name',
        message: 'Name for project',
        default: path.basename(process.cwd())
      },
      {
        name: 'app_description',
        message: 'Project Description',
        default: 'short description'
      },
      {
        type: 'confirm',
        name: 'sublime',
        message: 'Want to create a Sublime project? (This creates a sublime project file)'
      },
      {
        type: 'confirm',
        name: 'git',
        message: 'Want to create a git repository?',
        default: true
      }
    ];

    this.prompt(prompts, function (props) {
      props.app_name = _.kebabCase(props.app_name);

      var re = /-/gi;
      props.package_name = props.app_name.replace(re, '_');

      this.props = props;
      done();
    }.bind(this));
  },
  writing: {
    files: function () {

      // Create directories
      mkdirp.sync('vendor');

      // Copy files
      if (this.props.sublime) {
        this.copy('_sublime', this.props.app_name + '.sublime-project');
      }

      this.copy('_CHANGELOG.md', 'CHANGELOG.md');
      this.copy('_Gruntfile.js', 'Gruntfile.js');
      this.copy('editorconfig', '.editorconfig');
      this.copy('gitignore', '.gitignore');
    },
    templates: function () {

      // Main
      this.fs.copyTpl(this.templatePath('src/cmd/_main.go'),
        this.destinationPath('src/' + this.props.app_name + '/cmd/' + this.props.app_name + '/main.go'),
        {app_name: this.props.app_name, package_name: this.props.package_name});

      // Main test
      this.fs.copyTpl(this.templatePath('src/cmd/_main_test.go'),
        this.destinationPath('src/' + this.props.app_name + '/cmd/' + this.props.app_name + '/main_test.go'),
        {app_name: this.props.app_name, package_name: this.props.package_name});

      // Package
      this.fs.copyTpl(this.templatePath('src/_tmp.go'),
        this.destinationPath('src/' + this.props.app_name + '/' + this.props.app_name + '.go'),
        {package_name: this.props.package_name});

      // Package test
      this.fs.copyTpl(this.templatePath('src/_tmp_test.go'),
        this.destinationPath('src/' + this.props.app_name + '/' + this.props.app_name + '_test.go'),
        {package_name: this.props.package_name});

      // package.json
      this.fs.copyTpl(this.templatePath('_package.json'), this.destinationPath('package.json'),
        {app_name: this.props.app_name, app_description: this.props.app_description});

      // README.md
      this.fs.copyTpl(this.templatePath('_README.md'), this.destinationPath('README.md'),
        {app_name: this.props.app_name, app_description: this.props.app_description});

      // etc/install.cmd
      this.fs.copyTpl(this.templatePath('etc/_install.cmd'), this.destinationPath('etc/install.cmd'),
        {app_name: this.props.app_name});

      // etc/install.sh
      this.fs.copyTpl(this.templatePath('etc/_install.sh'), this.destinationPath('etc/install.sh'),
        {app_name: this.props.app_name});

      // etc/readme.txt
      this.fs.copyTpl(this.templatePath('etc/_readme.txt'), this.destinationPath('etc/readme.txt'),
        {app_name: this.props.app_name});

      // etc/uninstall.cmd
      this.fs.copyTpl(this.templatePath('etc/_uninstall.cmd'), this.destinationPath('etc/uninstall.cmd'),
        {app_name: this.props.app_name});

      // etc/uninstall.sh
      this.fs.copyTpl(this.templatePath('etc/_uninstall.sh'), this.destinationPath('etc/uninstall.sh'),
        {app_name: this.props.app_name});
    }
  },
  install: {
    npmInstall: function () {

      // Npm install without bower install
      this.npmInstall();
    },
    gitInit: function () {

      // Git init when confirm is true
      if (this.props.git) {
        var done = this.async();
        this.log('Create git repository for project.');

        this.spawnCommand('git', ['init'])
          .on('close', function () {
            done();
          });
      }
    },
    gitAdd: function () {

      // Git add -A when confirm is true
      if (this.props.git) {
        var done = this.async();

        this.spawnCommand('git', ['add', '-A'])
          .on('close', function () {
            done();
          });
      }
    },
    gitCommit: function () {

      // Git commit when confirm is true
      if (this.props.git) {

        var done = this.async();
        this.log('Git initial commit.');

        this.spawnCommand('git', ['commit', '-m', 'chore(lxgo): initial commit from generator'])
          .on('close', function () {
            done();
          });
      }
    },
    gb: function () {

      // Git commit when confirm is true
      if (this.props.git) {

        var done = this.async();
        this.log('Install gb for build and vendoring...');

        this.spawnCommand('go', ['get', 'github.com/constabulary/gb/...'])
          .on('close', function () {
            done();
          });
      }
    },
    gocov: function () {

      // Git commit when confirm is true
      if (this.props.git) {

        var done = this.async();
        this.log('Install gocov for coverage...');

        this.spawnCommand('go', ['get', 'github.com/axw/gocov/gocov'])
          .on('close', function () {
            done();
          });
      }
    },
    gocovHtml: function () {

      // Git commit when confirm is true
      if (this.props.git) {

        var done = this.async();
        this.log('Install gocov-html for coverage html reports...');

        this.spawnCommand('go', ['get', 'gopkg.in/matm/v1/gocov-html'])
          .on('close', function () {
            done();
          });
      }
    },
    gocovXtml: function () {

      // Git commit when confirm is true
      if (this.props.git) {

        var done = this.async();
        this.log('Install gocov-xml for coverage xml reports...');

        this.spawnCommand('go', ['get', 'github.com/AlekSi/gocov-xml'])
          .on('close', function () {
            done();
          });
      }
    }
  },
  end: function () {
    this.log('\nThe project has been successfully created!');
    this.log('Use the task grunt to install the vendor dependencies and run the application.\n');
    this.log('\tgrunt\n');
  }
});
