'use strict';

var path = require('path');
var _ = require('lodash');
var fs = require('fs');

module.exports = function (grunt) {

  // Load grunt tasks automatically
  require('load-grunt-tasks')(grunt);

  // Time how long tasks take. Can help when optimizing build times
  require('time-grunt')(grunt);

  // package.json
  var pkg = require('./package.json');
  var goPathDefault = process.env.GOPATH;

  console.log(goPathDefault);

  /**
   * Get compression mode for platform
   * @returns {*}
   */
  var getCompressExt = function () {

    if (process.platform === 'win32') {
      return '.zip';
    } else {
      return '.tar.gz';
    }
  };

  /**
   * Create the commands for coverage all packages and create reports
   * @param type Type of report
   * @returns {Array}
   */
  var getCoverageCommands = function (type) {

    type = type || 'html';

    var i, max;
    var res = [];
    var arr = pkg.coverPackages;

    // Add the modules for coverage
    for (i = 0, max = arr.length; i < max; i += 1) {
      // Must create file
      res[i] = 'go test -coverprofile ./tmp/' + i + '.part ' + arr[i];
    }

    // Save the parts
    res.push('echo mode: set >./tmp/coverage.out');

    if (process.platform === 'win32') {
      // Windows
      res.push('find /v "mode: set" tmp\\*.part | find ".go:" >> tmp\\coverage.out');
    } else {
      // Linux/Mac
      res.push('grep -h -v "mode: set" ./tmp/*.part >>./tmp/coverage.out');
    }

    // Converter
    var converter = 'gocov-' + type;

    // Add convert command
    res.push('gocov convert ./tmp/coverage.out | ' + converter + ' > ./_reports/cover_all.' + type);

    return res;
  };

  // Project configuration.
  grunt.initConfig({

    // Settings
    pkg: pkg,
    os: process.platform,
    arch: process.arch,
    suffix: process.platform + '-' + process.arch,

    // Tasks
    // Environment
    env: {
      default: {
        GOPATH: goPathDefault
      },
      dev: {
        GOPATH: __dirname + path.delimiter + path.join(__dirname, 'vendor')
      }
    },

    // Shell commands
    shell: {
      cmd: {
        command: function (cmd) {
          return cmd;
        }
      },
      cover: {
        command: function (type) {
          return getCoverageCommands(type).join('&&');
        }
      }
    },

    // Clean up directories
    clean: {
      vendor: ['vendor'],
      bin: ['bin'],
      dist: ['_dist'],
      pkg: ['_dist/*.tar.gz', '_dist/*.zip'],
      reports: ['_reports'],
      tmp: ['tmp'],
      cover: ['tmp', '_reports']
    },

    // Copy files
    copy: {
      dist: {
        options: {
          mode: true
        },
        files: [
          // Readme.md
          {src: ['etc/readme.txt'], dest: '_dist/<%= pkg.name %>/readme.txt'},

          // install.sh
          {src: ['etc/install.sh'], dest: '_dist/<%= pkg.name %>/install.sh'},

          // uninstall.sh
          {src: ['etc/uninstall.sh'], dest: '_dist/<%= pkg.name %>/uninstall.sh'},

          // install.cmd
          {src: ['etc/install.cmd'], dest: '_dist/<%= pkg.name %>/install.cmd'},

          // uninstall.cmd
          {src: ['etc/uninstall.cmd'], dest: '_dist/<%= pkg.name %>/uninstall.cmd'},

          // CHANGELOG.md
          {src: ['CHANGELOG.md'], dest: '_dist/<%= pkg.name %>/CHANGELOG.md'},

          // Bin directory
          {expand: true, cwd: 'bin/', src: ['**'], dest: '_dist/<%= pkg.name %>/bin/'}
        ]
      }
    },

    // Compress
    compress: {
      dist: {
        options: {
          archive: '_dist/<%= pkg.name %>.<%= pkg.version %>.<%= suffix %>' + getCompressExt()
        },
        files: [
          {expand: true, cwd: '_dist/<%= pkg.name %>/', src: ['**'], dest: '<%= pkg.name %>'}
        ]
      }
    },

    // Make directories for tasks
    mkdir: {
      cover: {
        options: {
          create: ['tmp', '_reports']
        }
      }
    },
    // Update CHANGELOG.md
    conventionalChangelog: {
      options: {
        changelogOpts: {
          preset: 'angular'
        }
      },
      release: {
        src: 'CHANGELOG.md'
      }
    },
    // Update version, changelog and commit
    bump: {
      options: {
        files: ['package.json'],
        updateConfigs: ['pkg'],
        commitFiles: ['.'],
        commitMessage: 'chore: release v%VERSION%',
        push: false
      }
    }
  });

  // Mapper for go commands
  // Run go with correct paths
  // Example: grunt go:"test ./src/..."
  grunt.registerTask('go', 'Mapper for go commands', function (args){

    // Check the go command args
    args = args || 'help';

    // go command
    var cmd = 'go ' + args;

    // Run tasks and command
    grunt.task.run('env:dev','shell:cmd:' + cmd, 'env:default');
  });

  // Helper task for open files
  // Using by open coverage report
  grunt.registerTask('open', 'Helper for open files', function (filePath) {

    // Default filepath
    filePath = filePath || null;

    // check filepath params
    if (!filePath) {
      grunt.log.error('Missing filePath for open.');
      return false;
    }

    // Check if out file exists for convert
    if (!fs.existsSync(filePath)) {
      grunt.log.error('File: ' + filePath + ' not found.');
      return false;
    }

    // Define Platform command for darwin
    var cmd = 'open';

    // Check platform
    if (process.platform === 'win32') {

      // Rewrite with windows start
      cmd = 'start';

    } else if (process.platform === 'linux') {

      // Rewrite with linux start
      cmd = 'xdg-open';
    }

    // Add filepath to command
    cmd += ' ' + filePath;

    // Run task
    grunt.task.run('shell:cmd:' + cmd);
  });

  // Backup or restore vendor manifest file
  // Example for backup: grunt vendor:backup
  // Example for restore: grunt vendor:restore
  grunt.registerTask('manifest', 'Backup or restore vendor manifest', function (option) {

    // Set vendor url
    option = option || null;
    var backupFile = path.join(__dirname, 'manifest.backup');
    var manifestFile = path.join(__dirname, 'vendor', 'manifest');

    // Check option
    if (option !== 'restore' && option !== 'backup') {
      grunt.log.error('Option must be backup or restore.');
      return false;
    }

    if (option === 'restore') {

      // Check file exists
      if (!fs.existsSync(backupFile)) {
        grunt.log.error('File: ' + backupFile + ' not found.');
        grunt.log.ok('Restore was not executed.');
        return true;
      }

      // Copy and delete backup file
      grunt.file.copy(backupFile, manifestFile);
      grunt.file.delete(backupFile);

      grunt.log.ok('Restore ' + backupFile);
      return true;

    }

    if (option === 'backup') {

      // Check file exists
      if (!fs.existsSync(manifestFile)) {
        grunt.log.error('File: ' + manifestFile + ' not found.');
        grunt.log.ok('Backup was not created.');
        return true;
      }

      // Copy and delete backup file
      grunt.file.copy(manifestFile, backupFile);

      grunt.log.ok('Backup ' + manifestFile);
      return true;
    }
  });

  // Test sources with coverage
  // Example for test all packages with coverage: grunt test
  // Example for test package without coverage: grunt test:"package-name-path"
  // Example for test package with coverage: grunt test:"package-name-path -cover"
  grunt.registerTask('test', 'Test the packages', function (args) {

    // Check args
    args = args || './src/... -cover';

    // Create test command
    var cmd = 'go test ' + args;

    grunt.task.run('format', 'env:dev', 'shell:cmd:' + cmd, 'env:default');
  });

  // Coverage packages
  // Usage: grunt cover:[package-name]:[report]
  // Example for coverage all packages with report on console: grunt cover
  // Example for coverage package with report on console: grunt cover:package-name
  // Example for coverage all packages with html report: grunt cover:html
  // Example for coverage package with html report: grunt cover:package-name:html
  grunt.registerTask('cover', 'Coverage packages', function (packageName, report) {

    // Set package name to default when not set
    packageName = packageName || null;
    report = report || null;

    // Check if report type in first argument
    if (packageName === 'html' || packageName === 'xml') {

      // Set report for coverage
      report = packageName;

      // Set package to null for coverage all packages
      packageName = null;
    }

    // Check if report argument != html | xml set null for no report
    if (report !== null && report !== 'html' && report !== 'xml') {
      report = null;
    }

    // Paths for reports
    var reportTmpFilePath = path.join('tmp', 'cover_package.out');
    var reportFilePath = path.join('_reports', 'cover_package.' + report);

    // Coverage with report on console for one package
    if (packageName && !report) {
      var cmdTest = 'gocov test ' + packageName + ' | gocov report';
      grunt.task.run('env:dev', 'shell:cmd:' + cmdTest, 'env:default');
      return true;
    }

    // Coverage with report in html or xml for one package
    if (packageName && report) {

      // Create coverage command
      var cmdReport = 'go test ' + packageName + ' -coverprofile ' + reportTmpFilePath;

      // Internal task to create and open report
      // Run after create coverprofile
      grunt.registerTask('createReport', 'Internal task open report', function () {

        // Check if exists out file for convert
        if (!fs.existsSync(reportTmpFilePath)) {
          grunt.log.ok('No reports generated...');
          return true;
        }

        // Converter binary
        var converter = 'gocov-' + report;

        // Create command for convert
        var cmdConvert = 'gocov convert ' + reportTmpFilePath + ' | ' + converter + ' > ' + reportFilePath;

        // Run tasks and open report
        if (report === 'html') {
          grunt.task.run('shell:cmd:' + cmdConvert, 'open:' + reportFilePath);
        } else {
          grunt.task.run('shell:cmd:' + cmdConvert);
        }
      });

      // Run cover tasks and open
      grunt.task.run('clean:cover', 'mkdir:cover', 'env:dev', 'shell:cmd:' + cmdReport, 'createReport', 'env:default');

      return true;
    }

    // Coverage with report on console for all packages
    if (!packageName && !report) {

      // Packages for coverage from package.json
      var coverPackages = pkg.coverPackages;

      var i, max, tmp = [];
      for (i = 0, max = coverPackages.length; i < max; i += 1) {

        // Add package coverage command
        tmp.push('gocov test ' + coverPackages[i] + ' | gocov report');
      }

      // Run cover tasks
      grunt.task.run('env:dev', 'shell:cmd:' + tmp.join('&&'), 'env:default');
      return true;
    }

    // Coverage with report in html or xml for all packages
    if (!packageName && report) {

      reportFilePath = path.join('_reports', 'cover_all.' + report);

      if (report === 'html') {
        grunt.task.run('clean:cover', 'mkdir:cover', 'env:dev', 'shell:cover:' + report, 'env:default', 'open:' + reportFilePath);
      } else {
        grunt.task.run('clean:cover', 'mkdir:cover', 'env:dev', 'shell:cover:' + report, 'env:default');
      }

      return true;
    }
  });

  // Format the code
  grunt.registerTask('format', 'Format the code', function () {

    // Format command
    var cmd = ['go fmt ./src/...', 'go vet ./src/...'].join('&&');

    // Run tasks and command
    grunt.task.run('env:dev','shell:cmd:' + cmd, 'env:default');
  });

  // Build distribution
  grunt.registerTask('build', 'Build distribution', function (flags) {

    // Build flags
    flags = flags || '';

    // Build command for gb
    var gbBuildCmd = 'gb build ' + flags;

    // Run tasks and command
    grunt.task.run('format', 'clean:bin', 'shell:cmd:' + gbBuildCmd, 'clean:dist', 'copy:dist');
  });

  // Run package with go run package-name/main.go
  // Example: grunt run:package-name
  grunt.registerTask('run', 'Run binary by name', function (binaryName) {

    // Set package name to default when not set
    binaryName = binaryName || pkg.name;

    // Create path to package main.go
    var binPath = path.join('bin', binaryName);

    // Command build and run binary
    var cmd = ['gb build', binPath].join('&&');

    // Run tasks and command
    grunt.task.run('format', 'shell:cmd:' + cmd);
  });

  // Compress the distribution for deploy
  grunt.registerTask('package', ['clean:pkg', 'compress:dist']);

  // Make a distribution release
  grunt.registerTask('release', 'Bump version, update changelog and tag version', function (version) {
    grunt.task.run([
      'shell:cmd:gb vendor restore',
      'test',
      'build',
      'bump:' + (version || 'patch') + ':bump-only',
      'conventionalChangelog',
      'bump-commit',
      'package'
    ]);
  });

  // Default task
  grunt.registerTask('default', ['shell:cmd:gb vendor restore', 'run']);
};
